#!/bin/bash

# Released under the terms of the WTFPL: http://wtfpl.net

# Write service files
write_services() {
[[ ! -e "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock.service" ]] && \
cat << EOF > "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock.service"
[Unit]
Description=Automatic announcement for sharps-talking-clock
                                                                                
[Service]
Type=simple
ExecStart=/usr/bin/sharps-talking-clock --chime
                                                                                
[Install]
WantedBy=user.target
EOF
[[ ! -e "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock-alarm.service" ]] && \
cat << EOF > "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock-alarm.service"
[Unit]
Description=Alarm for sharps-talking-clock
                                                                                
[Service]
Type=simple
ExecStart=/usr/bin/sharps-talking-clock --alarm
                                                                                
[Install]
WantedBy=user.target
EOF
    systemctl --user daemon-reload
}

# Write the timer
write_timer() {
    systemctl -q --user disable --now sharps-talking-clock.timer 2> /dev/null
    if [[ $1 -eq 1 ]]; then
    echo "Hourly announcements set to once an hour."
    time='0'
    elif [[ $1 -eq 4 ]]; then
    echo "Hourly announcements set to four times an hour."
        time='0/15'
    else
    echo "Hourly announcements off."
        exit 0
    fi
cat << EOF > "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock.timer"
[Unit]
Description=Sharps-talking-clock chimes
                                                                                                                                                                                                                  
[Timer]
AccuracySec=1us
OnCalendar=*:$time
                                                                                                                                                                                                                  
[Install]
WantedBy=timers.target
EOF
    systemctl --user daemon-reload
    systemctl -q --user enable --now sharps-talking-clock.timer
    exit 0
}

# Set the alarm
set_alarm() {
    systemctl -q --user disable --now sharps-talking-clock-alarm.timer 2> /dev/null
    if [[ "${1,,}" == "off" ]]; then
        echo "Alarm off."
        exit 0
    fi
cat << EOF > "${XDG_CONFIG_HOME:-$HOME/.config}/systemd/user/sharps-talking-clock-alarm.timer"
[Unit]
Description=Sharps-talking-clock alarm
                                                                                                                                                                                                                  
[Timer]
AccuracySec=1us
OnCalendar=$1
                                                                                                                                                                                                                  
[Install]
WantedBy=timers.target
EOF
    systemctl --user daemon-reload
    systemctl -q --user enable --now sharps-talking-clock-alarm.timer
    echo "Alarm set for $1."
    exit 0
}

# Play alarm sequence.
alarm() {
    for i in {1..3} ; do
        play -qV0 "${soundPath}announcement.ogg"
        command $0
        play -qV0 "${soundPath}alarm.ogg"
    done
    exit 0
}

# Announce with appropriate chimes.
chime() {
    if [[ "$minute" =~ 0|15|30|45 ]]; then
        play -qV0 "${soundPath}hour1.ogg"
    fi
    command $0
    if [[ "${minute}" == "00" ]]; then
        play -qV0 "${soundPath}hour2.ogg"
    fi
    exit 0
}

# Check for files
write_services

# Load settings file
source "${XDG_CONFIG_HOME:-$HOME/.config}/sharps-talking-clock.conf" 2> /dev/null

# Try to locate sounds.
if [[ -d "/usr/share/sounds/sharps-talking-clock" ]]; then
    soundPath="/usr/share/sounds/sharps-talking-clock/"
else
    soundPath="./sounds/"
fi

# 12 or 24 hour format, default 12
format=${format:-12}

if [[ $format -eq 12 ]]; then
    hour="$(date +'%-I')"
elif [[ $format -eq 24 ]]; then
    hour="$(date +'%-H')"
else
    echo "Invalid time format."
exit 1
fi
minute="$(date +'%M')"


# Handle cli args
case "$1" in
    "--alarm") alarm;;
    "--chime") chime;;
    "off") set_alarm off;;
esac
# Handle alarm setting cli args
if [[ "$1" =~ ^[01][0-9]:[0-5][0-9][AaPp][Mm]$ ]]; then
    time="${1::2}"
    time="${time#0}"
    if [[ "${1: -2}" == "AM" || "$1" == "am" ]]; then
        set_alarm "${1:: -2}"
    else
        set_alarm "$(($time + 12)):${1:3:2}"
    fi
fi
if [[ "$1" =~ ^[0-2][0-9]:[0-5][0-9]$ ]]; then
    set_alarm "$1"
fi


if [[ "$1" =~ ^[014]$ ]]; then
    write_timer $1
fi

# Time announcement
playString="${soundPath}its.ogg ${soundPath}"
# Fix for hours above 20.
if [[ $hour -le 20 ]]; then
    playString+="${hour}.ogg "
else
    playString+="20.ogg ${soundPath}${hour: -1}.ogg "
fi
playString+="${soundPath}"
case $minute in
    *0) playString+="${minute}.ogg";;
    0*) playString+="o.ogg ${soundPath}${minute: -1}.ogg";;
    1*) playString+="${minute}.ogg";;
    *) playString+="${minute::1}0.ogg ${soundPath}${minute: -1}.ogg";;
esac
if [[ $format -ne 24 ]]; then
    playString+=" ${soundPath}$(date +'%P'.ogg)"
fi

play -qV0 $playString 2> /dev/null

exit 0
